@@ -1,5 +1,5 @@
 /*  Lzip - LZMA lossless data compressor
-    Copyright (C) 2008-2015 Antonio Diaz Diaz.
+    Copyright (C) 2008-2018 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,26 +31,6 @@
 #include "decoder.h"
 
 
-const CRC32 crc32;
-
-
-void Pretty_print::operator()( const char * const msg ) const
-  {
-  if( verbosity >= 0 )
-    {
-    if( first_post )
-      {
-      first_post = false;
-      std::fprintf( stderr, "  %s: ", name_.c_str() );
-      for( unsigned i = 0; i < longest_name - name_.size(); ++i )
-        std::fputc( ' ', stderr );
-      if( !msg ) std::fflush( stderr );
-      }
-    if( msg ) std::fprintf( stderr, "%s\n", msg );
-    }
-  }
-
-
 /* Returns the number of bytes really read.
    If (returned value < size) and (errno == 0), means EOF was reached.
 */
@@ -97,6 +77,7 @@ bool Range_decoder::read_block()
     at_stream_end = ( stream_pos < buffer_size );
     partial_member_pos += pos;
     pos = 0;
+    show_dprogress();
     }
   return pos < stream_pos;
   }
@@ -110,7 +91,8 @@ void LZ_decoder::flush_data()
     crc32.update_buf( crc_, buffer + stream_pos, size );
     if( outfd >= 0 && writeblock( outfd, buffer + stream_pos, size ) != size )
       throw Error( "Write error" );
-    if( pos >= buffer_size ) { partial_data_pos += pos; pos = 0; }
+    if( pos >= dictionary_size )
+      { partial_data_pos += pos; pos = 0; pos_wrapped = true; }
     stream_pos = pos;
     }
   }
@@ -119,12 +101,12 @@ void LZ_decoder::flush_data()
 bool LZ_decoder::verify_trailer( const Pretty_print & pp ) const
   {
   File_trailer trailer;
-  const int trailer_size = File_trailer::size( member_version );
-  const unsigned long long member_size = rdec.member_position() + trailer_size;
+  int size = rdec.read_data( trailer.data, File_trailer::size );
+  const unsigned long long data_size = data_position();
+  const unsigned long long member_size = rdec.member_position();
   bool error = false;
 
-  int size = rdec.read_data( trailer.data, trailer_size );
-  if( size < trailer_size )
+  if( size < File_trailer::size )
     {
     error = true;
     if( verbosity >= 0 )
@@ -133,55 +115,59 @@ bool LZ_decoder::verify_trailer( const Pretty_print & pp ) const
       std::fprintf( stderr, "Trailer truncated at trailer position %d;"
                             " some checks may fail.\n", size );
       }
-    while( size < trailer_size ) trailer.data[size++] = 0;
+    while( size < File_trailer::size ) trailer.data[size++] = 0;
     }
 
-  if( member_version == 0 ) trailer.member_size( member_size );
-
-  if( !rdec.code_is_zero() )
-    {
-    error = true;
-    pp( "Range decoder final code is not zero." );
-    }
-  if( trailer.data_crc() != crc() )
+  const unsigned td_crc = trailer.data_crc();
+  if( td_crc != crc() )
     {
     error = true;
     if( verbosity >= 0 )
       {
       pp();
-      std::fprintf( stderr, "CRC mismatch; trailer says %08X, data CRC is %08X\n",
-                    trailer.data_crc(), crc() );
+      std::fprintf( stderr, "CRC mismatch; stored %08X, computed %08X\n",
+                    td_crc, crc() );
       }
     }
-  if( trailer.data_size() != data_position() )
+  const unsigned long long td_size = trailer.data_size();
+  if( td_size != data_size )
     {
     error = true;
     if( verbosity >= 0 )
       {
       pp();
-      std::fprintf( stderr, "Data size mismatch; trailer says %llu, data size is %llu (0x%llX)\n",
-                    trailer.data_size(), data_position(), data_position() );
+      std::fprintf( stderr, "Data size mismatch; stored %llu (0x%llX), computed %llu (0x%llX)\n",
+                    td_size, td_size, data_size, data_size );
       }
     }
-  if( trailer.member_size() != member_size )
+  const unsigned long long tm_size = trailer.member_size();
+  if( tm_size != member_size )
     {
     error = true;
     if( verbosity >= 0 )
       {
       pp();
-      std::fprintf( stderr, "Member size mismatch; trailer says %llu, member size is %llu (0x%llX)\n",
-                    trailer.member_size(), member_size, member_size );
+      std::fprintf( stderr, "Member size mismatch; stored %llu (0x%llX), computed %llu (0x%llX)\n",
+                    tm_size, tm_size, member_size, member_size );
       }
     }
-  if( !error && verbosity >= 2 && data_position() > 0 && member_size > 0 )
-    std::fprintf( stderr, "%6.3f:1, %6.3f bits/byte, %5.2f%% saved.  ",
-                  (double)data_position() / member_size,
-                  ( 8.0 * member_size ) / data_position(),
-                  100.0 * ( 1.0 - ( (double)member_size / data_position() ) ) );
-  if( !error && verbosity >= 4 )
-    std::fprintf( stderr, "data CRC %08X, data size %9llu, member size %8llu.  ",
-                  trailer.data_crc(), trailer.data_size(), trailer.member_size() );
-  return !error;
+  if( error ) return false;
+  if( verbosity >= 2 )
+    {
+    if( verbosity >= 4 ) show_header( dictionary_size );
+    if( data_size == 0 || member_size == 0 )
+      std::fputs( "no data compressed.  ", stderr );
+    else
+      std::fprintf( stderr, "%6.3f:1, %5.2f%% ratio, %5.2f%% saved.  ",
+                    (double)data_size / member_size,
+                    ( 100.0 * member_size ) / data_size,
+                    100.0 - ( ( 100.0 * member_size ) / data_size ) );
+    if( verbosity >= 4 ) std::fprintf( stderr, "CRC %08X, ", td_crc );
+    if( verbosity >= 3 )
+      std::fprintf( stderr, "decompressed %9llu, compressed %8llu.  ",
+                    data_size, member_size );
+    }
+  return true;
   }
 
 
@@ -197,7 +183,7 @@ int LZ_decoder::decode_member( const Pretty_print & pp )
   Bit_model bm_rep2[State::states];
   Bit_model bm_len[State::states][pos_states];
   Bit_model bm_dis_slot[len_states][1<<dis_slot_bits];
-  Bit_model bm_dis[modeled_distances-end_dis_model];
+  Bit_model bm_dis[modeled_distances-end_dis_model+1];
   Bit_model bm_align[dis_align_size];
   Len_model match_len_model;
   Len_model rep_len_model;
@@ -213,25 +199,23 @@ int LZ_decoder::decode_member( const Pretty_print & pp )
     const int pos_state = data_position() & pos_state_mask;
     if( rdec.decode_bit( bm_match[state()][pos_state] ) == 0 )	// 1st bit
       {
-      const uint8_t prev_byte = peek_prev();
-      if( state.is_char() )
-        {
-        state.set_char1();
-        put_byte( rdec.decode_tree8( bm_literal[get_lit_state(prev_byte)] ) );
-        }
+      Bit_model * const bm = bm_literal[get_lit_state(peek_prev())];
+      if( state.is_char_set_char() )
+        put_byte( rdec.decode_tree8( bm ) );
       else
-        {
-        state.set_char2();
-        put_byte( rdec.decode_matched( bm_literal[get_lit_state(prev_byte)],
-                                       peek( rep0 ) ) );
-        }
+        put_byte( rdec.decode_matched( bm, peek( rep0 ) ) );
       }
     else					// match or repeated match
       {
       int len;
       if( rdec.decode_bit( bm_rep[state()] ) != 0 )		// 2nd bit
         {
-        if( rdec.decode_bit( bm_rep0[state()] ) != 0 )		// 3rd bit
+        if( rdec.decode_bit( bm_rep0[state()] ) == 0 )		// 3rd bit
+          {
+          if( rdec.decode_bit( bm_len[state()][pos_state] ) == 0 ) // 4th bit
+            { state.set_short_rep(); put_byte( peek( rep0 ) ); continue; }
+          }
+        else
           {
           unsigned distance;
           if( rdec.decode_bit( bm_rep1[state()] ) == 0 )	// 4th bit
@@ -247,34 +231,28 @@ int LZ_decoder::decode_member( const Pretty_print & pp )
           rep1 = rep0;
           rep0 = distance;
           }
-        else
-          {
-          if( rdec.decode_bit( bm_len[state()][pos_state] ) == 0 ) // 4th bit
-            { state.set_short_rep(); put_byte( peek( rep0 ) ); continue; }
-          }
         state.set_rep();
         len = min_match_len + rdec.decode_len( rep_len_model, pos_state );
         }
       else					// match
         {
-        const unsigned rep0_saved = rep0;
         len = min_match_len + rdec.decode_len( match_len_model, pos_state );
-        const int dis_slot = rdec.decode_tree6( bm_dis_slot[get_len_state(len)] );
-        if( dis_slot < start_dis_model ) rep0 = dis_slot;
-        else
+        unsigned distance = rdec.decode_tree6( bm_dis_slot[get_len_state(len)] );
+        if( distance >= start_dis_model )
           {
+          const unsigned dis_slot = distance;
           const int direct_bits = ( dis_slot >> 1 ) - 1;
-          rep0 = ( 2 | ( dis_slot & 1 ) ) << direct_bits;
+          distance = ( 2 | ( dis_slot & 1 ) ) << direct_bits;
           if( dis_slot < end_dis_model )
-            rep0 += rdec.decode_tree_reversed( bm_dis + rep0 - dis_slot - 1,
-                                               direct_bits );
+            distance += rdec.decode_tree_reversed(
+                        bm_dis + ( distance - dis_slot ), direct_bits );
           else
             {
-            rep0 += rdec.decode( direct_bits - dis_align_bits ) << dis_align_bits;
-            rep0 += rdec.decode_tree_reversed4( bm_align );
-            if( rep0 == 0xFFFFFFFFU )		// marker found
+            distance +=
+              rdec.decode( direct_bits - dis_align_bits ) << dis_align_bits;
+            distance += rdec.decode_tree_reversed4( bm_align );
+            if( distance == 0xFFFFFFFFU )		// marker found
               {
-              rep0 = rep0_saved;
               rdec.normalize();
               flush_data();
               if( len == min_match_len )	// End Of Stream marker
@@ -294,9 +272,9 @@ int LZ_decoder::decode_member( const Pretty_print & pp )
               }
             }
           }
-        rep3 = rep2; rep2 = rep1; rep1 = rep0_saved;
+        rep3 = rep2; rep2 = rep1; rep1 = rep0; rep0 = distance;
         state.set_match();
-        if( rep0 >= dictionary_size || rep0 >= data_position() )
+        if( rep0 >= dictionary_size || ( rep0 >= pos && !pos_wrapped ) )
           { flush_data(); return 1; }
         }
       copy_block( rep0, len );
