@@ -1,5 +1,5 @@
 /*  Lzip - LZMA lossless data compressor
-    Copyright (C) 2008-2015 Antonio Diaz Diaz.
+    Copyright (C) 2008-2018 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -30,8 +30,12 @@ public:
     static const int next[states] = { 0, 0, 0, 0, 1, 2, 3, 4, 5, 6, 4, 5 };
     st = next[st];
     }
-  void set_char1()     { st -= ( st < 4 ) ? st : 3; }	// for st < 7
-  void set_char2()     { st -= ( st < 10 ) ? 3 : 6; }	// for st >= 7
+  bool is_char_set_char()
+    {
+    if( st < 7 ) { st -= ( st < 4 ) ? st : 3; return true; }
+    else { st -= ( st < 10 ) ? 3 : 6; return false; }
+    }
+  void set_char_rep()  { st = 8; }
   void set_match()     { st = ( st < 7 ) ? 7 : 10; }
   void set_rep()       { st = ( st < 7 ) ? 8 : 11; }
   void set_short_rep() { st = ( st < 7 ) ? 9 : 11; }
@@ -43,7 +47,9 @@ enum {
   min_dictionary_size = 1 << min_dictionary_bits,	// >= modeled_distances
   max_dictionary_bits = 29,
   max_dictionary_size = 1 << max_dictionary_bits,
+  min_member_size = 36,
   literal_context_bits = 3,
+  literal_pos_state_bits = 0,				// not used
   pos_state_bits = 2,
   pos_states = 1 << pos_state_bits,
   pos_state_mask = pos_states - 1,
@@ -107,17 +113,22 @@ struct Len_model
   };
 
 
-class Pretty_print
+// defined in main.cc
+extern int verbosity;
+
+class Pretty_print		// requires global var 'int verbosity'
   {
   std::string name_;
+  std::string padded_name;
   const char * const stdin_name;
   unsigned longest_name;
   mutable bool first_post;
 
 public:
-  explicit Pretty_print( const std::vector< std::string > & filenames )
+  Pretty_print( const std::vector< std::string > & filenames )
     : stdin_name( "(stdin)" ), longest_name( 0 ), first_post( false )
     {
+    if( verbosity <= 0 ) return;
     const unsigned stdin_name_len = std::strlen( stdin_name );
     for( unsigned i = 0; i < filenames.size(); ++i )
       {
@@ -132,6 +143,9 @@ public:
     {
     if( filename.size() && filename != "-" ) name_ = filename;
     else name_ = stdin_name;
+    padded_name = "  "; padded_name += name_; padded_name += ": ";
+    if( name_.size() < longest_name )
+      padded_name.append( longest_name - name_.size(), ' ' );
     first_post = true;
     }
 
@@ -165,14 +179,21 @@ public:
   void update_buf( uint32_t & crc, const uint8_t * const buffer,
                    const int size ) const
     {
+    uint32_t c = crc;
     for( int i = 0; i < size; ++i )
-      crc = data[(crc^buffer[i])&0xFF] ^ ( crc >> 8 );
+      c = data[(c^buffer[i])&0xFF] ^ ( c >> 8 );
+    crc = c;
     }
   };
 
 extern const CRC32 crc32;
 
 
+inline bool isvalid_ds( const unsigned dictionary_size )
+  { return ( dictionary_size >= min_dictionary_size &&
+             dictionary_size <= max_dictionary_size ); }
+
+
 inline int real_bits( unsigned value )
   {
   int bits = 0;
@@ -194,8 +215,22 @@ struct File_header
   bool verify_magic() const
     { return ( std::memcmp( data, magic_string, 4 ) == 0 ); }
 
+  bool verify_prefix( const int sz ) const	// detect (truncated) header
+    {
+    for( int i = 0; i < sz && i < 4; ++i )
+      if( data[i] != magic_string[i] ) return false;
+    return ( sz > 0 );
+    }
+  bool verify_corrupt() const			// detect corrupt header
+    {
+    int matches = 0;
+    for( int i = 0; i < 4; ++i )
+      if( data[i] == magic_string[i] ) ++matches;
+    return ( matches > 1 && matches < 4 );
+    }
+
   uint8_t version() const { return data[4]; }
-  bool verify_version() const { return ( data[4] <= 1 ); }
+  bool verify_version() const { return ( data[4] == 1 ); }
 
   unsigned dictionary_size() const
     {
@@ -207,20 +242,17 @@ struct File_header
 
   bool dictionary_size( const unsigned sz )
     {
-    if( sz >= min_dictionary_size && sz <= max_dictionary_size )
+    if( !isvalid_ds( sz ) ) return false;
+    data[5] = real_bits( sz - 1 );
+    if( sz > min_dictionary_size )
       {
-      data[5] = real_bits( sz - 1 );
-      if( sz > min_dictionary_size )
-        {
-        const unsigned base_size = 1 << data[5];
-        const unsigned fraction = base_size / 16;
-        for( int i = 7; i >= 1; --i )
-          if( base_size - ( i * fraction ) >= sz )
-            { data[5] |= ( i << 5 ); break; }
-        }
-      return true;
+      const unsigned base_size = 1 << data[5];
+      const unsigned fraction = base_size / 16;
+      for( unsigned i = 7; i >= 1; --i )
+        if( base_size - ( i * fraction ) >= sz )
+          { data[5] |= ( i << 5 ); break; }
       }
-    return false;
+    return true;
     }
   };
 
@@ -231,8 +263,7 @@ struct File_trailer
 			//  4-11 size of the uncompressed data
 			// 12-19 member size including header and trailer
 
-  static int size( const int version = 1 )
-    { return ( ( version >= 1 ) ? 20 : 12 ); }
+  enum { size = 20 };
 
   unsigned data_crc() const
     {
@@ -273,17 +304,38 @@ struct Error
   };
 
 
+const char * const bad_magic_msg = "Bad magic number (file not in lzip format).";
+const char * const bad_dict_msg = "Invalid dictionary size in member header.";
+const char * const corrupt_mm_msg = "Corrupt header in multimember file.";
+const char * const trailing_msg = "Trailing data not allowed.";
+
 // defined in decoder.cc
 int readblock( const int fd, uint8_t * const buf, const int size );
 int writeblock( const int fd, const uint8_t * const buf, const int size );
 
+// defined in list.cc
+int list_files( const std::vector< std::string > & filenames,
+                const bool ignore_trailing, const bool loose_trailing );
+
 // defined in main.cc
-extern int verbosity;
+struct stat;
+const char * bad_version( const unsigned version );
+const char * format_ds( const unsigned dictionary_size );
+void show_header( const unsigned dictionary_size );
+int open_instream( const char * const name, struct stat * const in_statsp,
+                   const bool no_ofile, const bool reg_only = false );
 void show_error( const char * const msg, const int errcode = 0,
                  const bool help = false );
+void show_file_error( const char * const filename, const char * const msg,
+                      const int errcode = 0 );
 void internal_error( const char * const msg );
 class Matchfinder_base;
-void show_progress( const unsigned long long partial_size = 0,
-                    const Matchfinder_base * const m = 0,
-                    const Pretty_print * const p = 0,
-                    const unsigned long long cfile_size = 0 );
+void show_cprogress( const unsigned long long cfile_size = 0,
+                     const unsigned long long partial_size = 0,
+                     const Matchfinder_base * const m = 0,
+                     const Pretty_print * const p = 0 );
+class Range_decoder;
+void show_dprogress( const unsigned long long cfile_size = 0,
+                     const unsigned long long partial_size = 0,
+                     const Range_decoder * const d = 0,
+                     const Pretty_print * const p = 0 );
